/*
 * ----------------------------------------------------------------------
 * This file is part of the Microsoft TinyCLR SDK Code Samples.
 *
 * Copyright (C) 2004 Microsoft Corporation.  All rights reserved.
 *
 * This source code is intended only as a supplement to Microsoft
 * Development Tools and/or on-line documentation.  See those other
 * materials for detailed information regarding Microsoft code samples.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 * ----------------------------------------------------------------------
 * 
 * Led.cs
 * 
 * SUMMARY
 *   This is a driver for a single LED.
 */

using System;
using System.Threading;
using System.Collections;
using System.Runtime.CompilerServices;

using Microsoft.SPOT.Hardware;


namespace Microsoft.SPOT.Drivers.Digi.GPIO
{

    public class Led : IDisposable
    {
        protected OutputPort m_outputPort;

        //--//

        public Led(Cpu.Pin pin, bool defaultState)
        {
            m_outputPort = new OutputPort(pin, defaultState);
        }

        public Led(Cpu.Pin pin)
            : this(pin, false)
        {
        }

        public bool On
        {
            get
            {
                return m_outputPort.Read();
            }
            set
            {
                m_outputPort.Write(value);
            }
        }

        public void Dispose()
        {
            m_outputPort.Dispose();
        }
    }


    public class BlinkingLed : Led
    {
        internal class Request
        {
            public int m_blinkInterval;
            public int m_blinkCount;

            internal Request(int blinkInterval, int blinkCount)
            {
                m_blinkInterval = blinkInterval;
                m_blinkCount = blinkCount;
            }
        }

        //--//

        private Thread m_worker;
        private Request m_request;
        private bool m_blinking;

        //--//

        public BlinkingLed(Cpu.Pin pin, bool defaultState)
            : base(pin, defaultState)
        {
            m_worker = null;
        }

        [MethodImplAttribute(MethodImplOptions.Synchronized)]
        public void Blink(int blinkInterval, int blinkCount)
        {
            if (m_worker != null)
            {
                m_blinking = false;

                m_worker = null;
            }

            m_request = new Request(blinkInterval, blinkCount);

            m_worker = new Thread(new ThreadStart(this.CountedBlinkThreadProc));

            m_worker.Start();
        }

        [MethodImplAttribute(MethodImplOptions.Synchronized)]
        public void Blink(int blinkInterval)
        {
            if (m_worker != null)
            {
                m_blinking = false;

                m_worker = null;
            }

            m_request = new Request(blinkInterval, -1);

            m_worker = new Thread(new ThreadStart(this.BlinkThreadProc));

            m_blinking = true;

            m_worker.Start();
        }

        [MethodImplAttribute(MethodImplOptions.Synchronized)]
        public void StopBlink()
        {
            if (m_worker != null)
            {
                m_blinking = false;

                m_worker = null;
            }

            m_outputPort.Write(false);
        }

        private void CountedBlinkThreadProc()
        {
            Request r = m_request;

            bool currentState = m_outputPort.Read();

            while (--r.m_blinkCount >= 0)
            {
                DoBlink(currentState, r.m_blinkInterval);
            }
        }

        private void BlinkThreadProc()
        {
            Request r = m_request;

            bool currentState = m_outputPort.Read();

            while (m_blinking)
            {
                DoBlink(currentState, r.m_blinkInterval);
            }
        }

        private void DoBlink(bool state, int interval)
        {
            m_outputPort.Write(!state);

            Thread.Sleep(interval);

            m_outputPort.Write(state);

            Thread.Sleep(interval);
        }
    }

}
